/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-edit.h"

void openldev_edit_update_textview (OpenLDev*);

// Cut the currently selected text from the buffer. Then add the text to the
// actionmanager of the current buffer and redo the syntax highlighting
void openldev_edit_cut (GtkMenuItem *menuitem,
                        gpointer data) 
{
  OpenLDev *openldev = (OpenLDev*) data;
  GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (openldev->window));
  
  if (GTK_IS_TEXT_VIEW (widget))
  {
    GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
    gtk_text_buffer_cut_clipboard (GTK_TEXT_BUFFER (buffer), gtk_clipboard_get (GDK_NONE), TRUE);
    openldev_edit_update_textview (openldev);
  }
}

// Cut the currently selected text from the buffer.
void openldev_edit_copy (GtkMenuItem *menuitem,
                         gpointer data) 
{
  OpenLDev *openldev = (OpenLDev*) data;
  GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (openldev->window));
  
  if (GTK_IS_TEXT_VIEW (widget))
  {
    GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
    gtk_text_buffer_copy_clipboard (GTK_TEXT_BUFFER (buffer), 
                                    gtk_clipboard_get (GDK_NONE));
  }
  else if (GTK_IS_TREE_VIEW (widget))
  {
    GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
    GtkTreeModel *model;
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected (selection, &model, &iter))
    {
      gchar *str;
      string text;
      
      for (int i=0; i < gtk_tree_model_get_n_columns (model); i++)
        if (gtk_tree_model_get_column_type (model, i) == G_TYPE_STRING)
        {
          gtk_tree_model_get (model, &iter, i, &str, -1);
          text += string (str) + " ";
        }
      
      if (text.length() > 0)
        text.erase (text.length () - 1, 1);
      
      gtk_clipboard_set_text (gtk_clipboard_get (GDK_NONE), text.c_str(), text.length());
      g_free (str);
    }
  }
}

// Paste the clipboard text to the current cursor position. Then add the text 
// to the actionmanager of the current buffer and redo the syntax highlighting
void openldev_edit_paste (GtkMenuItem *menuitem,
                          gpointer data) 
{
  OpenLDev *openldev = (OpenLDev*) data;
  GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (openldev->window));
  
  if (GTK_IS_TEXT_VIEW (widget))
  {
    GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
    gtk_text_buffer_paste_clipboard (GTK_TEXT_BUFFER (buffer), 
                                     gtk_clipboard_get (GDK_NONE), NULL, TRUE);
    openldev_edit_update_textview (openldev);
  }
}

// Select all text in the text_view
void openldev_edit_select_all (GtkMenuItem *menuitem,
                               gpointer data) 
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev_file_manager_get_file_open (openldev->notebook))
  {
    GtkTextIter start, end;
    GtkTextBuffer *buffer = GTK_TEXT_BUFFER (openldev_file_manager_get_textbuffer 
                                             (openldev->notebook));
    gtk_text_buffer_get_bounds (buffer, &start, &end);
    gtk_text_buffer_select_range (buffer, &start, &end);
  }
}

// Deselect any text that is selected
void openldev_edit_deselect (GtkMenuItem *menuitem,
                             gpointer data) 
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev_file_manager_get_file_open (openldev->notebook))
  {
    GtkSourceBuffer *buffer = openldev_file_manager_get_textbuffer (openldev->notebook);
    GtkTextMark *mark = gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (buffer));
    GtkTextIter iter;
    
    gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (buffer), &iter, mark);
    gtk_text_buffer_move_mark (GTK_TEXT_BUFFER (buffer),
                               gtk_text_buffer_get_selection_bound (GTK_TEXT_BUFFER (buffer)),
                               &iter);
  }
}

// Undo the last action that was performed. If there was no action, then don't
// do anything. To undo a delete, just insert the text. To undo an insert, the
// text must be removed from the specified offset & then backwards the length
void openldev_edit_undo (GtkMenuItem *menuitem,
                         gpointer data) 
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev_file_manager_get_file_open (openldev->notebook))
  {
    GtkSourceBuffer *buffer = openldev_file_manager_get_textbuffer (openldev->notebook);
    if (gtk_source_buffer_can_undo (buffer))
      gtk_source_buffer_undo (buffer);
  }
}

// Redo the last action that was undone. If there was no action, then don't
// do anything. To redo an insert, you must remove the text from the specified
// offset to backwards the length of the text. To redo a delete, just move
// backward the length of the string & insert it
void openldev_edit_redo (GtkMenuItem *menuitem,
                         gpointer data) 
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev_file_manager_get_file_open (openldev->notebook))
  {
    GtkSourceBuffer *buffer = openldev_file_manager_get_textbuffer (openldev->notebook);
    if (gtk_source_buffer_can_redo (buffer))
      gtk_source_buffer_redo (buffer);
  }
}

// Load the environment settings dialog and then store the changes that the user
// makes and rewrite the environment settings file. Then update the fonts.
void openldev_edit_preferences (GtkMenuItem *menuitem,
                                gpointer data) 
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  EnvironmentDialog *dialog = openldev_env_dialog_new ();
  openldev_env_dialog_set_load_on_launch (dialog, openldev->env->auto_load);
  openldev_env_dialog_set_undo (dialog, openldev->env->undo);
  openldev_env_dialog_set_bracket (dialog, openldev->env->brackets);
  openldev_env_dialog_set_autosave (dialog, openldev->env->auto_save);
  openldev_env_dialog_set_tab_size (dialog, openldev->env->tab_size);
  openldev_env_dialog_set_autoindent (dialog, openldev->env->auto_indent);
  openldev_env_dialog_set_replace_tabs (dialog, openldev->env->replace_tabs);
  openldev_env_dialog_set_right_margin (dialog, openldev->env->right_margin);
  openldev_env_dialog_set_split_words (dialog, openldev->env->wrap_split);
  openldev_env_dialog_set_wrap (dialog, openldev->env->wrap);
  openldev_env_dialog_set_error (dialog, openldev->env->error.c_str());
  openldev_env_dialog_set_warning (dialog, openldev->env->warning.c_str());
  openldev_env_dialog_set_font (dialog, openldev->env->font_face.c_str(), 
                                openldev->env->font_size.c_str());
  
  switch (openldev_env_dialog_run (dialog))
  {
    case GTK_RESPONSE_OK:
    {
      // Change properties by reference & destroy
      openldev_environment_set_font_face (openldev->env, openldev_env_dialog_get_font_face(dialog));
      openldev_environment_set_font_size (openldev->env, openldev_env_dialog_get_font_size(dialog));
      openldev_environment_set_replace_tabs (openldev->env, openldev_env_dialog_get_replace_tabs(dialog));
      openldev_environment_set_auto_indent (openldev->env, openldev_env_dialog_get_autoindent(dialog));
      openldev_environment_set_undo (openldev->env, openldev_env_dialog_get_undo(dialog));
      openldev_environment_set_wrap (openldev->env, openldev_env_dialog_get_wrap(dialog));
      openldev_environment_set_wrap_split (openldev->env, openldev_env_dialog_get_split_words(dialog));
      openldev_environment_set_brackets (openldev->env, openldev_env_dialog_get_bracket(dialog));
      openldev_environment_set_error (openldev->env, openldev_env_dialog_get_error(dialog));
      openldev_environment_set_warning (openldev->env, openldev_env_dialog_get_warning(dialog));
      openldev_environment_set_auto_save (openldev->env, openldev_env_dialog_get_autosave(dialog));
      openldev_environment_set_tab_size (openldev->env, openldev_env_dialog_get_tab_size(dialog));
      openldev_environment_set_auto_load (openldev->env, openldev_env_dialog_get_load_on_launch(dialog));
      openldev_environment_set_right_margin (openldev->env, openldev_env_dialog_get_right_margin(dialog));
      
      // Reset fonts for all tabs
      gchar *font_description = g_strconcat (openldev->env->font_face.c_str(), " ",
                                             openldev->env->font_size.c_str(), NULL);
      PangoFontDescription *fd = pango_font_description_from_string (font_description); 
      openldev_file_manager_modify_fonts (openldev->notebook, fd);
      openldev_file_manager_set_properties (openldev->notebook, TRUE, openldev->env);
      g_free (font_description);
      break;
    }
    default:
      break;
  }

  gtk_widget_destroy (dialog->dialog);
  delete dialog;
}

// Update the TextBuffer that is sent to this function. It marks it modified
// when text has been entered and shows the cursor on screen.
void openldev_edit_update_textview (OpenLDev *openldev)
{
  int current = gtk_notebook_get_current_page (GTK_NOTEBOOK(openldev->notebook->notebook));
  openldev_file_manager_mark_current_tab_modified (openldev->notebook, TRUE, current);
  openldev_file_manager_scroll_to_mark (openldev->notebook);
}
