/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-auto-list.h"

// Create a new list of autocomplete words
AutoList* openldev_auto_list_new ()
{
  AutoList *list = new AutoList;
  list->autolist = g_completion_new (NULL);
  return list;
}

// Get one to three choices that would match the user's text
GList* openldev_auto_list_get_matches (AutoList *list, 
                                       gchar *match)
{
  gchar *new_prefix;
  GList *glist = g_completion_complete_utf8 (list->autolist, match, &new_prefix);
  return glist;
}

// Make sure the character is acceptable for a word
gboolean openldev_auto_list_check_char (char ch)
{
  if (isalnum(ch))
    return TRUE;
  else if (ch != '_')
    return FALSE;

  return TRUE;
}

// Erase all of the words
void openldev_auto_list_clean_all (AutoList *list)
{ g_completion_clear_items (list->autolist);
}

// Add all of the words in all of the source and header files
void openldev_auto_list_parse_file (AutoList *list, gchar *file)
{
  gchar *contents;
  string text, word;
  GError *error;
  gsize length;
  GList *words = NULL;
  int chr;
  
  if (g_file_get_contents (file, &contents, &length, &error))
  {
    text = contents;
    // Go through every character of the file
    while (text.length() >  0)
    {
      // If the character is still acceptable, add it
      chr = 0;
      while (openldev_auto_list_check_char (text[chr]))
        chr++;
      word = text.substr (0, chr);
      
      // If the word is of correct length and add it
      if (word.length() >= 4 && openldev_auto_list_check_char (word[0]))
        words = g_list_prepend (words, (gpointer) g_strdup ((gchar*) word.c_str()));

      // Erase the used character
      text.erase (0, chr + 1);
    }
  }
  
  g_completion_add_items (list->autolist, words);
  g_list_free (words);
  g_free (contents);
}

// Parse a whole list of files
void openldev_auto_list_parse_files (AutoList *list, 
                                     GList *fns)
{
  for (unsigned int i=0; i < g_list_length (fns); i++)
    openldev_auto_list_parse_file (list, (gchar*) g_list_nth_data (fns, i));
}
