/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef OPENLDEV_ENVIRONMENT_H
#define OPENLDEV_ENVIRONMENT_H

#include <gconf/gconf-client.h>
#include <cstdlib>
#include <glib.h>
#include <iostream>
#include <fstream>
#include <vector>
using namespace std;

typedef struct _EnvironmentSettings  EnvironmentSettings;

struct _EnvironmentSettings
{
  GConfClient *client;

  int auto_save;
  int browser_width;
  int height;
  int outputmanager_height;
  int tab_size;
  int undo;
  int width;
  
  gboolean auto_indent;
  gboolean auto_load;
  gboolean brackets;
  gboolean line_numbers;
  gboolean replace_tabs;
  gboolean right_margin;
  gboolean wrap;
  gboolean wrap_split;
  
  string error;
  string font_face;
  string font_size;
  string last_project;
  string warning;
  
  vector<string> recent_files;
};

EnvironmentSettings* openldev_environment_settings_new ();

void openldev_environment_set_auto_indent  (EnvironmentSettings*, gboolean);
void openldev_environment_set_auto_load    (EnvironmentSettings*, gboolean);
void openldev_environment_set_brackets     (EnvironmentSettings*, gboolean);
void openldev_environment_set_line_numbers (EnvironmentSettings*, gboolean);
void openldev_environment_set_replace_tabs (EnvironmentSettings*, gboolean);
void openldev_environment_set_right_margin (EnvironmentSettings*, gboolean);
void openldev_environment_set_wrap         (EnvironmentSettings*, gboolean);
void openldev_environment_set_wrap_split   (EnvironmentSettings*, gboolean);

void openldev_environment_set_auto_save            (EnvironmentSettings*, int);
void openldev_environment_set_browser_width        (EnvironmentSettings*, int);
void openldev_environment_set_height               (EnvironmentSettings*, int);
void openldev_environment_set_outputmanager_height (EnvironmentSettings*, int);
void openldev_environment_set_tab_size             (EnvironmentSettings*, int);
void openldev_environment_set_undo                 (EnvironmentSettings*, int);
void openldev_environment_set_width                (EnvironmentSettings*, int);

void openldev_environment_set_error        (EnvironmentSettings*, const char*);
void openldev_environment_set_font_face    (EnvironmentSettings*, const char*);
void openldev_environment_set_font_size    (EnvironmentSettings*, const char*);
void openldev_environment_set_last_project (EnvironmentSettings*, const char*);
void openldev_environment_set_warning      (EnvironmentSettings*, const char*);

void openldev_environment_add_recent_file (EnvironmentSettings*, const char*);
const char* openldev_environment_get_recent_short (EnvironmentSettings*, int);

#endif
