/***************************************************************************
 *   Copyright (C) 2005-2006 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "makefile-editor.h"

void makefile_editor_edit_clicked (GtkButton*, gpointer);
void makefile_editor_refresh (GtkButton*, gpointer);

gchar *file   = g_strconcat (INSTALL_PREFIX, "/share/pixmaps/openldev/mfeditor-file.png", NULL);
gchar *dir    = g_strconcat (INSTALL_PREFIX, "/share/pixmaps/openldev/mfeditor-directory.png", NULL);
gchar *target = g_strconcat (INSTALL_PREFIX, "/share/pixmaps/openldev/mfeditor-target.png", NULL);
gchar *script = g_strconcat (INSTALL_PREFIX, "/share/pixmaps/openldev/mfeditor-script.png", NULL);
gchar *var    = g_strconcat (INSTALL_PREFIX, "/share/pixmaps/openldev/mfeditor-variable.png", NULL);

extern "C" 
{
// Initialize the plugin
G_MODULE_EXPORT PluginState init (gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  MakefileEditor *editor = makefile_editor_init (openldev);
  editor->fullname = "";
  editor->project = openldev->project;
  
  g_timeout_add (2000, GSourceFunc (on_update_makefile_editor), (gpointer) editor);
  g_signal_connect (G_OBJECT (editor->content), "button-press-event",
                    G_CALLBACK (makefile_editor_button_press), (gpointer) editor);
  g_signal_connect (G_OBJECT (editor->edit), "clicked",
                    G_CALLBACK (makefile_editor_edit_clicked), (gpointer) editor);
  g_signal_connect (G_OBJECT (editor->add), "clicked",
                    G_CALLBACK (makefile_editor_add), (gpointer) editor);
  g_signal_connect (G_OBJECT (editor->refresh), "clicked",
                    G_CALLBACK (makefile_editor_refresh), (gpointer) editor);
  
  return PLUGIN_OK;
} 

// Destroy the plugin. This currently has no use because OpenLDev
// will never destroy it. This is for future use.
G_MODULE_EXPORT PluginState destroy (gpointer data)
{
  openldev_error ("Destroying Plugin!");
  return PLUGIN_OK;
}
};

// Create the Makefile Editor
MakefileEditor* makefile_editor_init (OpenLDev *openldev)
{
  MakefileEditor *editor = new MakefileEditor;

  editor->vbox = gtk_vbox_new (FALSE, 3);
  editor->hbox = gtk_hbox_new (FALSE, 1);
  editor->scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  editor->content = gtk_tree_view_new ();
  editor->project = openldev->project;
  editor->add = gtk_button_new_with_label ("Add");
  editor->edit = gtk_button_new_with_label ("Edit");
  editor->refresh = gtk_button_new_with_label ("Refresh");
  
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (editor->content), FALSE);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (editor->scrolledwindow),
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_add (GTK_CONTAINER (editor->scrolledwindow), editor->content);
  
  gtk_box_pack_start (GTK_BOX (editor->hbox), editor->add, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (editor->hbox), editor->edit, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (editor->hbox), editor->refresh, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (editor->vbox), editor->hbox, FALSE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (editor->vbox), editor->scrolledwindow, TRUE, TRUE, 0);
  
  gtk_widget_show_all (editor->vbox);
  openldev_browser_add_tab (openldev->browser, editor->vbox, gtk_label_new ("Project"));
  
  return editor;
}

// Handle button press events in the Makefile Editor content treeview
gboolean makefile_editor_button_press (GtkWidget *widget, 
                                   GdkEventButton *event, 
                                   gpointer data)
{
  MakefileEditor *editor = (MakefileEditor*) data;

  if (event->type == GDK_2BUTTON_PRESS)
  {
    GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
    GtkTreeModel *model;
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected (selection, &model, &iter))
    {
      gchar *directory;
      int type;
      gtk_tree_model_get (model, &iter, 1, &directory, 2, &type, -1);
      
      // Move one directory up in the Makefile structure
      if (type == MAKEFILE_EDITOR_MOVE_UP)
      {
        string temp = editor->current_path, temp2;
        temp.erase (temp.length () - 1, 1);
        
        while (temp.find ("/", 0) != string::npos)
        {
          temp2 += temp.substr (0, temp.find ("/", 0) + 1);
          temp.erase (0, temp.find ("/", 0) + 1);
        }
        
        editor->current_path = g_strdup ((gchar*) temp2.c_str());
        temp = temp2.substr (0, temp2.length() - 1);
        while (temp.find ("/", 0) != string::npos)
          temp.erase (0, temp.find ("/", 0) + 1);
        editor->current_dir = g_strdup ((gchar*) temp.c_str());
        makefile_editor_populate (editor);
      }
      // Move into the directory of a subproject
      else if (type == MAKEFILE_EDITOR_SUBPROJECT)
      {
        editor->current_dir = directory;
        editor->current_path = g_strconcat (editor->current_path.c_str(), directory, "/", NULL);
        makefile_editor_populate (editor);
      }
      // Otherwise, edit the element. This can be done with the Edit button as well
      else
        makefile_editor_edit (editor);
    }
    
    return TRUE;
  }
  
  return FALSE;
}

// Refresh the content of the editor
void makefile_editor_refresh (GtkButton *button,
                              gpointer data)
{ makefile_editor_populate ((MakefileEditor*) data);
}

// The edit button was pressed, so we need to edit the element
void makefile_editor_edit_clicked (GtkButton *button,
                                   gpointer data)
{ makefile_editor_edit ((MakefileEditor*) data);
}

// Add all of the information for the current Makefile.am
void makefile_editor_populate (MakefileEditor *editor)
{
  unsigned int i, j;
  GtkTreeIter row, child, childchild;
  GtkTreeModel *model = GTK_TREE_MODEL (gtk_tree_store_new (3, GDK_TYPE_PIXBUF, 
                        G_TYPE_STRING, G_TYPE_INT));
  gtk_tree_view_set_model (GTK_TREE_VIEW (editor->content), model);
  
  while (g_list_length(gtk_tree_view_get_columns (GTK_TREE_VIEW (editor->content))) > 0)
    gtk_tree_view_remove_column (GTK_TREE_VIEW (editor->content),
      gtk_tree_view_get_column (GTK_TREE_VIEW (editor->content), 0));
  
  if (editor->fullname.empty())
    return;
  
  gchar *makefile = g_strconcat (editor->project->location.c_str(), 
                                 editor->current_path.c_str(),
                                 "Makefile.am", NULL);
  Makefile *mf = openldev_makefile_parser_new (makefile);
  GList *targets = makefile_editor_get_targets (mf);
  
  // Create the columns and add them
  GtkCellRenderer *irenderer = gtk_cell_renderer_pixbuf_new ();
  GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
	GtkTreeViewColumn *icolumn = gtk_tree_view_column_new_with_attributes ("",
                               irenderer, "pixbuf", 0, NULL);
	GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes ("Content",
                              renderer, "text", 1, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (editor->content), icolumn);
  gtk_tree_view_append_column (GTK_TREE_VIEW (editor->content), column);
  
  // Add the top directory & the '..' directory if needed
  gtk_tree_store_append (GTK_TREE_STORE (model), &row, NULL);
  if (!editor->current_path.empty())
  {
    gtk_tree_store_set (GTK_TREE_STORE (model), &row, 0, gtk_image_get_pixbuf (GTK_IMAGE 
                        (gtk_image_new_from_file (dir))), 1, editor->current_dir.c_str(),
                        2, MAKEFILE_EDITOR_BASE, -1);
    makefile_editor_add_row (model, child, row, gtk_image_get_pixbuf (GTK_IMAGE
                             (gtk_image_new_from_file (dir))), "..",
                             MAKEFILE_EDITOR_MOVE_UP);
  }
  else
    gtk_tree_store_set (GTK_TREE_STORE (model), &row, 0, gtk_image_get_pixbuf (GTK_IMAGE 
                        (gtk_image_new_from_file (dir))), 1, "/",
                        2, MAKEFILE_EDITOR_BASE, -1);
  
  // List all of the sub directories in the project
  for (i=0; i < mf->types.size(); i++)
    if (g_ascii_strcasecmp (mf->names[i].c_str(), "SUBDIRS") == 0)
      for (j=0; j < mf->attributes[i]->attr.size(); j++)
        makefile_editor_add_row (model, child, row, gtk_image_get_pixbuf (GTK_IMAGE
                                 (gtk_image_new_from_file (dir))), mf->attributes[i]->attr[j].c_str(),
                                 MAKEFILE_EDITOR_SUBPROJECT);
  
  // List all of the targets in the project
  for (i=0; i < g_list_length (targets); i++)
  {
    makefile_editor_add_row (model, child, row, gtk_image_get_pixbuf (GTK_IMAGE
                             (gtk_image_new_from_file (target))), (gchar*) g_list_nth_data 
                             (targets, i), MAKEFILE_EDITOR_TARGET);
    
    string target = (gchar*) g_list_nth_data (targets, i);
    int location = target.find (".", 0);
    if (location != (signed) string::npos)
    {
      target.erase (location, 1);
      target.insert (location, "_");
    }
    
    GList *sources = makefile_editor_get_sources (mf, (gchar*) target.c_str());
    for (j=0; j < g_list_length (sources); j++)
      makefile_editor_add_row (model, childchild, child, gtk_image_get_pixbuf (GTK_IMAGE
                               (gtk_image_new_from_file (file))), (gchar*) g_list_nth_data 
                               (sources, j), MAKEFILE_EDITOR_FILE);
  }
  
  // List all of the scripts and variables in the project
  for (i=0; i < mf->types.size(); i++)
  {
    if (mf->types[i] == MAKEFILE_SCRIPT)
      makefile_editor_add_row (model, child, row, gtk_image_get_pixbuf (GTK_IMAGE
                               (gtk_image_new_from_file (script))), mf->names[i].c_str(),
                               MAKEFILE_EDITOR_SCRIPT);
    else if (makefile_editor_is_variable (mf, i))
      makefile_editor_add_row (model, child, row, gtk_image_get_pixbuf (GTK_IMAGE 
                               (gtk_image_new_from_file (var))), mf->names[i].c_str(),
                               MAKEFILE_EDITOR_VARIABLE);
  }
  
  gtk_tree_view_expand_row (GTK_TREE_VIEW (editor->content),
                            gtk_tree_path_new_from_string ("0"),
                            FALSE);
  
  g_list_free (targets);
  g_free (makefile);
}

// Add a row to the Makefile Editor content GtkTreeView
void makefile_editor_add_row (GtkTreeModel *model,
                              GtkTreeIter &child,
                              GtkTreeIter &row,
                              GdkPixbuf *pixbuf,
                              const char *text,
                              int id)
{
  gtk_tree_store_append (GTK_TREE_STORE (model), &child, &row);
  gtk_tree_store_set (GTK_TREE_STORE (model), &child,
                      0, pixbuf, 1, text, 2, id, -1);
}

// If all of these terms are met, then the Makefile Editor treats the
// current item as a variable
gboolean makefile_editor_is_variable (Makefile *mf,
                                  int i)
{
  return (mf->names[i] != "INCLUDES" &&
          mf->names[i] != "EXTRA_DIST" &&
          mf->names[i] != "AM_CXXFLAGS" &&
          mf->names[i] != "AM_CFLAGS" &&
          mf->names[i] != "SUBDIRS" &&
          mf->names[i].find ("_PROGRAMS", 0) == string::npos &&
          mf->names[i].find ("_LIBRARIES", 0) == string::npos &&
          mf->names[i].find ("_LTLIBRARIES", 0) == string::npos &&
          mf->names[i].find ("_LDFLAGS", 0) == string::npos &&
          mf->names[i].find ("_LDADD", 0) == string::npos &&
          mf->names[i].find ("_LIBADD", 0) == string::npos &&
          mf->names[i].find ("_DATA", 0) == string::npos &&
          mf->names[i].find ("_SOURCES", 0) == string::npos &&
          mf->names[i].find ("dir", 0) == string::npos &&
          mf->types[i] != MAKEFILE_SCRIPT &&
          mf->types[i] != MAKEFILE_COMMENT &&
          mf->types[i] != MAKEFILE_STRING);
}

// Get a list of the sources in a target
GList* makefile_editor_get_sources (Makefile *mf,
                                    string target)
{
  unsigned int i, j;
  GList *sources = NULL;
 
  target += "_SOURCES";
  for (i=0; i < mf->types.size(); i++)
    if (mf->names[i] == target)
      for (j=0; j < mf->attributes[i]->attr.size(); j++)
        sources = g_list_append (sources, (gpointer) mf->attributes[i]->attr[j].c_str());
  
  return sources;
}

// Get a list of targets in the Makefile.am
GList* makefile_editor_get_targets (Makefile *mf)
{
  GList *targets = NULL;
  GList *prefixes = NULL;
  GList *final = NULL;
  unsigned int i, j;
  string temp;
  
  // Get a list of targets of programs, libraries & data
  for (i=0; i < mf->types.size(); i++)
  {
    if (mf->names[i].find ("_PROGRAMS", 0) != string::npos ||
        mf->names[i].find ("_LIBRARIES", 0) != string::npos ||
        mf->names[i].find ("_LTLIBRARIES", 0) != string::npos)
      for (j=0; j < mf->attributes[i]->attr.size(); j++)
      {
        string tmp = mf->names[i].substr (0, mf->names[i].find ("_", 0));
        prefixes = g_list_prepend (prefixes, (gpointer) tmp.c_str());
        targets = g_list_append (targets, (gpointer) mf->attributes[i]->attr[j].c_str());
      }
    else if (mf->names[i] == "EXTRA_DIST")
      for (j=0; j < mf->attributes[i]->attr.size(); j++)
      {
        temp = mf->attributes[i]->attr[j];
        temp.erase (temp.find ("_DATA)", 0), 6);
        temp.erase (temp.find ("$(", 0), 2);
        targets = g_list_append (targets, (gpointer) g_strdup ((gchar*) temp.c_str()));
      }
  }
  
  // Make sure that there are no repeats, meaning that a target is also a prefix
  // of a program or library
  for (i=0; i < g_list_length (targets); i++)
  {
    gboolean add = TRUE;
    gchar *temp = (gchar*) g_list_nth_data (targets, i);
    for (j=0; j < g_list_length (prefixes); j++)
      if (g_strcasecmp (temp, (gchar*) g_list_nth_data (prefixes, j)) == 0)
        add = FALSE;
    
    if (add)
      final = g_list_append (final, (gpointer) g_strdup 
                             ((gchar*) g_list_nth_data (targets, i)));
  }
  
  g_list_free (prefixes);
  g_list_free (targets);
  return final;
}

// Check if the current project for which the Makefile is being shown is still
// the project that is open
gboolean on_update_makefile_editor (gpointer data)
{
  MakefileEditor *editor = (MakefileEditor*) data;

  if (editor->project->fullname != editor->fullname)
  {
    editor->fullname = editor->project->fullname;
    editor->current_dir = "/";
    editor->current_path = "";
    makefile_editor_populate (editor);
  }
  else if (editor->project->fullname.empty())
  {
    editor->fullname = "";
    editor->current_dir = "";
    editor->current_path = "";
    makefile_editor_populate (editor);
  }
  
  return TRUE;
}
